local options = {
    type='group',
    args = {
        collect = {
            type = 'toggle',
            name = 'Data collection',
            desc = 'Toggles the collection of latency data',
            get = "IsCollectData",
            set = "ToggleCollectData",
        },
        interval = {
            type = 'range',
            name = 'Collection interval',
            desc = 'Sets the interval of data collection (in seconds)',
            get = "GetCollectInterval",
            set = "SetCollectInterval",
            min = 30,
            max = 600,
            step = 30,
        },
        report = {
            type = 'execute',
			name = 'Print report',
            desc = 'Outputs a latency report to the main chat window',
            func = "ReportData",
        },
    },
}

local serverName = GetRealmName()

LatencyStats = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceEvent-2.0")

LatencyStats:RegisterChatCommand("/latency", "/lsa", options)

LatencyStats:RegisterDB("LatencyStatsDB")
LatencyStats:RegisterDefaults("profile", {
    collect = true,
    interval = 30,
} )

LatencyStats:RegisterDefaults("account", {
	servers = {
		['*'] = {
			samples = 0,
			avgLatency = 0,
		}
	}
} )



function LatencyStats:OnEnable()
    LatencyStats:ScheduleCollection()
end

function LatencyStats:ScheduleCollection()
   if self.db.profile.collect then
        LatencyStats:CollectData()
		self:ScheduleRepeatingEvent("LatencyStats_CollectData", LatencyStats.CollectData, self.db.profile.interval)
	else
	    self:CancelAllScheduledEvents()
	end
end

function LatencyStats:CollectData()
	-- LatencyStats:Print(GetRealmName()..","..date("%Y-%m-%d %H:%M:%S")..","..select(3,GetNetStats()))
 	LatencyStats.db.account.servers[serverName].samples = LatencyStats.db.account.servers[serverName].samples + 1
 	LatencyStats.db.account.servers[serverName].avgLatency = ((LatencyStats.db.account.servers[serverName].avgLatency * (LatencyStats.db.account.servers[serverName].samples -1)) + select(3,GetNetStats())) / LatencyStats.db.account.servers[serverName].samples
 	-- SendChatMessage("Average latency is "..LatencyStats.db.account.serverName.avgLatency.."ms over "..LatencyStats.db.account.serverName.samples.." samples." , "WHISPER", nil, UnitName("player"))
end

function LatencyStats:ReportData()
   local reportName
   for savedName in pairs(LatencyStats.db.account.servers) do
        if savedName == serverName then
            reportName = ">>>"..savedName
		else
		    reportName = savedName
		end
   		LatencyStats:Print(reportName..": "..LatencyStats.db.account.servers[savedName].samples.." samples, "..math.ceil(LatencyStats.db.account.servers[savedName].avgLatency).."ms")
   end
end

--[[ Functions for getting and setting options ]]

function LatencyStats:IsCollectData()
	return self.db.profile.collect
end

function LatencyStats:ToggleCollectData()
	self.db.profile.collect = not self.db.profile.collect
	LatencyStats:ScheduleCollection()
end

function LatencyStats:GetCollectInterval()
	return self.db.profile.interval
end

function LatencyStats:SetCollectInterval(newValue)
	self.db.profile.interval = newValue
	LatencyStats:ScheduleCollection()
end